#!/usr/bin/env python3
'''
Accept options from command line and make access copy.
Use bitc.py -h for help
'''
import argparse
import subprocess
import sys
import os
from ififuncs import hashlib_md5
import ififuncs

def getffprobe(variable, streamvalue, which_file):
    '''
    Returns a specific ffprobe technical metadata value.
    This is a good candidate to be moved to ififuncs.
    '''
    variable = subprocess.check_output([
        'ffprobe',
        '-v', 'error',
        '-select_streams', 'v:0',
        '-show_entries',
        streamvalue,
        '-of', 'default=noprint_wrappers=1:nokey=1',
        which_file
    ])
    return variable.decode(sys.stdout.encoding)


def set_options(args_):
    '''
    Parse command line options.
    '''
    parser = argparse.ArgumentParser(
        description='IFI Irish Film Institute H264 FFMPEG Encoder.'
        ' Written by Kieran O\'Leary.'
    )
    parser.add_argument(
        'input'
    )
    parser.add_argument(
        '-clean',
        action='store_true',
        help='Disables watermark and timecode for a clean image'
    )
    parser.add_argument(
        '-watermark',
        action='store_true',
        help='Disables timecode and only displays watermark'
    )
    parser.add_argument(
        '-timecode',
        action='store_true',
        help='Disables watermark and only displays timecode'
    )
    parser.add_argument(
        '-yadif',
        action='store_true',
        help='Yet Another DeInterlace Filter'
    )
    parser.add_argument(
        '-middle',
        action='store_true',
        help='Put timecode in the middle'
    )
    parser.add_argument(
        '-crf',
        help='Set quality. Default is 23, lower number ='
        ' large file/high quality, high number = small file/poor quality'
    )
    parser.add_argument(
        '-o',
        help='Set output directory.'
        'The default directory is the same directory as input.'
    )
    parser.add_argument(
        '-logo',
        help='Full path to a transparent PNG that will be '
        'overlaid in the top right corner.'
    )
    parser.add_argument(
        '-scale',
        help='Rescale video.'
        ' Usage: -scale 1920x1080 or -scale 720x576 etc'
    )
    parser.add_argument(
        '-md5',
        action='store_true',
        help='Get md5 sidecar for your output file'
    )
    parser.add_argument(
        '-map',
        action='store_true',
        help='Force default mapping, eg. 1 audio/video stream'
    )

    parser.add_argument(
        '-player',
        action='store_true',
        help='uses yadif, 4:3 DAR with 1:1 PAR, with no watermark or timecode.'
    )
    parser.add_argument(
        '-wide',
        action='store_true',help='Adds 16:9 metadata flag'
    )
    parsed_args = parser.parse_args(args_)
    return parsed_args


def build_filter(args, filename):
    '''
    Builds the ffmpeg -vf filtergraph, if needed.
    '''
    h264_options = []
    filter_list = []
    filtergraph = ''
    if args.yadif:
        h264_options.append('yadif')
    if args.logo:
        h264_options.append('overlay=main_w-overlay_w-5:5')
    if args.scale:
        h264_options.append('scale=%s' % args.scale)
        # width_height = args.scale
    if args.player:
        args.clean = True
        args.yadif = True
    if not args.clean:
        drawtext_option = setup_drawtext(args, filename)
        h264_options.append(drawtext_option)
    for option in h264_options:
        filtergraph += option + ','
    if len(filtergraph) > 0:
        if filtergraph[-1] == ',':
            filtergraph = filtergraph[:-1]
        filter_list = ['-filter_complex', filtergraph]
        print(filter_list)
    return filter_list


def get_filenames(args):
    '''
    Get information about filenames, paths etc.
    '''
    cli_input = args.input
    # Input, either file or firectory, that we want to process.
    # Store the directory containing the input file/directory.
    wd = os.path.dirname(cli_input)
    video_files = []
    if wd == '':
        cli_input = os.path.join(os.getcwd(), args.input)
    # Check if input is a file.
    # AFAIK, os.path.isfile only works if full path isn't present.
    if os.path.isfile(cli_input):
        video_files.append(cli_input)  # Add filename to list
    # Check if input is a directory.
    elif os.path.isdir(cli_input):
        for files in os.listdir(cli_input):
            if files.endswith(('.mov', '.mp4', '.mxf', '.mkv', '.avi')):
                if files[0] != '.':
                    video_files.append(os.path.join(cli_input, files))
    # Prints some stuff if input isn't a file or directory.
    else:
        print("Your input isn't a file or a directory.")
    return video_files


def setup_drawtext(args, filename):
    '''
    Sets up the filtergraphs for either timecode, watermark or both.
    '''
    # HDV m2t streams report two height values, so the rsplit() just accepts the first.
    video_height = float(getffprobe('video_height', 'stream=height', filename).rsplit()[0])
    # Calculate appropriate font size
    font_size = video_height / 12
    watermark_size = video_height / 14
    if sys.platform == "darwin":
        font_path = "fontfile=/Library/Fonts/AppleGothic.ttf"
    elif sys.platform.startswith("linux"):
        font_path = "fontfile=/usr/share/fonts/truetype/freefont/FreeSerifBold.ttf"
    elif sys.platform == "win32":
        font_path = "'fontfile=C\:\\\Windows\\\Fonts\\\\'arial.ttf'"
    # Get starting timecode
    timecode_test_raw = getffprobe(
        'timecode_test_raw',
        'format_tags=timecode:stream_tags=timecode',
        filename
    )
    framerate = getffprobe(
        'get_frame_rate',
        'stream=avg_frame_rate',
        filename
    ).rsplit()[0]
    # This tests if there is actually a timecode present in the file.
    if not timecode_test_raw:
        # The timecode needs to be phrased in a way unique to each O.S.
        # Note the backslashes.
        # This section makes up a timecode if none is present in the file.
        if sys.platform == "darwin" or sys.platform.startswith("linux"):
            timecode_test = '01\\\:00\\\:00\\\:00'
        elif sys.platform == "win32":
            timecode_test = '01\:00\:00\:00'
    else:
        # If timecode is present, this will escape the colons
        # so that it is compatible with each operating system.
        if sys.platform == "darwin" or sys.platform.startswith("linux"):
            timecode_test = timecode_test_raw.replace(':', '\\\:').rstrip()
        elif sys.platform == "win32":
            timecode_test = timecode_test_raw.replace(':', '\\:').rstrip()
    # This removes the new line character from the framemrate.
    if args.middle:
        timecode_option = "drawtext=%s:fontcolor=white:fontsize=%s:timecode=%s:rate=%s:boxcolor=0x000000AA:box=1:x=(w-text_w)/2:y=(h-text_h)/2" % (font_path, font_size, timecode_test, framerate)
        watermark_option = "drawtext=%s:fontcolor=white:text='IFI IRISH FILM ARCHIVE':x=(w-text_w)/2:y=h/1.2:fontsize=%s:alpha=0.4"  % (font_path, watermark_size)
    else:
        timecode_option = "drawtext=%s:fontcolor=white:fontsize=%s:timecode=%s:rate=%s:boxcolor=0x000000AA:box=1:x=(w-text_w)/2:y=h/1.2" % (font_path, font_size, timecode_test, framerate)
        watermark_option = "drawtext=%s:fontcolor=white:text='IFI IRISH FILM ARCHIVE':x=(w-text_w)/2:y=(h-text_h)/2:fontsize=%s:alpha=0.4"  % (font_path, watermark_size)
    bitc_watermark = timecode_option + ',' + watermark_option
    if args.timecode:
        return timecode_option
    elif args.watermark:
        return watermark_option
    else:
        return bitc_watermark


def main(args_):
    '''
    Launch the various functions that will make a h264/mp4 access copy.
    '''
    ififuncs.check_existence(['ffprobe', 'ffmpeg'])
    args = set_options(args_)
    video_files = get_filenames(args)
    for filename in video_files:
        filter_list = build_filter(args, filename)
        make_h264(filename, args, filter_list)


def make_h264(filename, args, filter_list):
    '''
    Launches the actuall ffmpeg process using all the info gleaned so far.
    '''
    if args.crf:
        crf_value = args.crf
    else:
        crf_value = '23'
    if args.o:
        output = args.o + '/' + os.path.basename(filename) + "_h264.mov"
    else:
        output = filename + "_h264.mov"
    ffmpeg_args = [
        'ffmpeg',
        '-i', filename,
    ]
    if args.logo:
        ffmpeg_args.extend(['-i', args.logo])
    ffmpeg_args += [
        '-c:a', 'aac',
        '-c:v', 'libx264',
        '-pix_fmt', 'yuv420p',
        '-crf', crf_value
    ]
    if args.wide:
        ffmpeg_args.append('-aspect')
        ffmpeg_args.append('16:9')
    if not args.map:
        ffmpeg_args.append('-map')
        ffmpeg_args.append('0:a?')
        ffmpeg_args.append('-map')
        ffmpeg_args.append('0:v')
    if len(filter_list) > 0:
        for _filter in filter_list:
            ffmpeg_args.append(_filter)
    ffmpeg_args.append(output)
    print(ffmpeg_args)
    subprocess.call(ffmpeg_args)
    if args.md5:
        manifest = '%s_manifest.md5' % filename
        print('Generating md5 sidecar...')
        h264_md5 = hashlib_md5(filename)
        with open(manifest, 'wb') as fo:
            fo.write('%s  %s' % (h264_md5, filename))

if __name__ == "__main__":
    main(sys.argv[1:])




#!/usr/bin/env python3
import sys
from setuptools import setup

if sys.version_info < (3, 8):
    print("Python 3.8 or higher is required - earlier python 3 versions may work but were not tested.")
    sys.exit(1)
setup(
    author='Kieran O\'Leary',
    author_email='kieran.o.leary@gmail.com',
    description="Scripts for processing moving image material in the Irish Film Institute/Irish Film Archive",
    long_description=("""\
Scripts for use in the IFI Irish Film Archive. Scripts have been tested in OSX/Windows 7/10 and Ubuntu 18.04. The aim is to make cross-platform scripts, but please get in touch with any issues. It is best to download all scripts, as some of them share code.

Most scripts take either a file or a directory as their input, for example makeffv1.py filename.mov or premis.py path/to/folder_of_stuff. (It's best to just drag and drop the folder or filename into the terminal)

Note: Documentation template has been copied from mediamicroservices

NOTE: Objects.py has been copied from https://github.com/simsong/dfxml. walk_to_dfxml.py has also been copied but has been customised in order to add command line arguments for optionally turning off checksum generation. For more context, see https://github.com/simsong/dfxml/pull/28
"""),
    scripts=[
        'scripts/Objects.py',
        'scripts/accession.py',
        'scripts/accession_register.py',
        'scripts/as11fixity.py',
        'scripts/batchaccession.py',
        'scripts/batchsipcreator.py',
        'scripts/bitc.py',
        'scripts/concat.py',
        'scripts/copyit.py',
        'scripts/dcpaccess.py',
        'scripts/dcpfixity.py',
        'scripts/deletefiles.py',
        'scripts/dfxml.py',
        'scripts/durationcheck.py',
        'scripts/ffv1mkvvalidate.py',
        'scripts/framemd5.py',
        'scripts/ififuncs.py',
        'scripts/loopline_repackage.py',
        'scripts/make_mediaconch.py',
        'scripts/makedfxml.py',
        'scripts/makedip.py',
        'scripts/makeffv1.py',
        'scripts/makepbcore.py',
        'scripts/makeuuid.py',
        'scripts/makezip.py',
        'scripts/manifest.py',
        'scripts/masscopy.py',
        'scripts/massqc.py',
        'scripts/mergepbcore.py',
        'scripts/multicopy.py',
        'scripts/normalise.py',
        'scripts/order.py',
        'scripts/package_update.py',
        'scripts/packagecheck.py',
        'scripts/prores.py',
        'scripts/seq2ffv1.py',
        'scripts/sipcreator.py',
        'scripts/strongbox_fixity.py',
        'scripts/subfolders.py',
        'scripts/testfiles.py',
        'scripts/validate.py',
        'scripts/walk_to_dfxml.py'
    ],
    license='MIT',
    install_requires=[
        'lxml',
        'bagit',
        'dicttoxml',
        'future',
        'clairmeta'
    ],
    data_files=[('', ['film_scan_aip_documentation.txt', '26_XYZ-22_Rec709.cube'])],
    include_package_data=True,
    name='ifiscripts',
    version='2022.10.19.1',
    python_requires='>=3.8'
)



#!/usr/bin/env python3
'''
Audits logfiles to determine the parent of a derivative package.
This script can aid in automating large accessioning procedures that involve
the accessioning of derivatives along with masters, eg a Camera Card and
a concatenated derivative, or a master file and a mezzanine.

order.py will be able to determine if something is a reproduction,
but it won't be able to tell if acquisitions are deposits, purchases or donations.
So leave it blank in the latter instances.
'''
import sys
import os
import ififuncs

def file_count(source):
    '''
    Checks how many mov/mp4 files are in a dir.
    '''
    count = 0
    for root, _, filenames in os.walk(source):
        for files in filenames:
            if files.lower().endswith(('.mov', '.mp4')):
                count += 1
    return count


def main(args):
    '''
    Analyzes a directory containing Object Entry packages and returns their
    parent or lack thereof.
    '''
    source = args
    if os.path.basename(source)[:2] == 'oe' or os.path.basename(source)[:3] == 'aaa' :
        oe_uuid_dict = ififuncs.group_ids(os.path.dirname(source))
        for root, _, filenames in os.walk(source):
            for filename in filenames:
                if filename.endswith('_sip_log.log'):
                    uuid_search = ififuncs.find_parent(
                        os.path.join(root, filename), oe_uuid_dict
                    )
                    if 'not a child' in uuid_search:
                        # Checks if a single AV file is in the objects dir.
                        uuid_dir = os.path.join(os.path.dirname(root))
                        if file_count(os.path.join(uuid_dir, 'objects')) == 1:
                            print('%s has no parent but this could be because it is a single file' % os.path.basename(os.path.dirname(uuid_dir)))
                            proceed = ififuncs.ask_yes_no('add %s to accession list?' % os.path.basename(os.path.dirname(uuid_dir)))
                            if proceed == 'Y':
                                print(os.path.basename(os.path.dirname(uuid_dir)))
                                return os.path.basename(os.path.dirname(uuid_dir))
                        else:
                            # master
                            return None
                    elif 'has a parent' in uuid_search:
                        parent = uuid_search.split()[-1]
                        # Commenting this out for now - this just adds the dash really.
                        # print parent[:2].upper() + '-' + parent[2:]
                        print(parent)
                        return parent

if __name__ == '__main__':
    main(sys.argv[1])



#!/usr/bin/env python

import os
import sys
from glob import glob

input = sys.argv[1]
if not os.path.isdir(input):
    print 'Error - Not a directory - Please provide a directory as input. Exiting...'
    sys.exit()
else:
    os.chdir(input)

    tiff_check = glob('*.tiff')
    dpx_check = glob('*.dpx')
    if len(dpx_check) > 0:
        images = dpx_check

    elif len(tiff_check) > 0:
        images = tiff_check
    else:
        print 'no images found'


permission = ''
for i in images:
    new_filename = ''
    split_names = i.split('_')
    if 'oe' in split_names[0]:
        for x in split_names[1:-1]:
            new_filename += x + '_'
        new_filename += split_names[-1]
        if not permission == 'y' or permission == 'Y':
            permission =  raw_input('\n**** Original filename = %s\n**** New filename =  %s\n**** If this looks ok, please press Y, otherwise, type N\n' % ( i, new_filename))
            while permission not in ('Y','y','N','n'):
                permission =  raw_input('\n**** Original filename = %s\n**** New filename =  %s\n**** If this looks ok, please press Y, otherwise, type N\n' % ( i, new_filename))
            if permission == 'n' or permission == 'N':
                print 'Exiting at your command'
                sys.exit()
            elif permission =='y' or permission == 'Y':
                os.rename(i, new_filename)
                print '**** Renaming %s with %s' % (i, new_filename)
        elif permission == 'y' or permission == 'Y':
            os.rename(i, new_filename)
            print '**** Renaming %s with %s' % (i, new_filename)   
    else:
        print 'This does not need to be renamed - exiting...'
        sys.exit()
            
  


#!/usr/bin/env python3
'''
Launches multiple copyit jobs. This is different to masscopy.py,
which takes a single directory as input and launches multiple copyit jobs.
This script takes multiple inputs and copies them to the output directory.
Make God have mercy on us all
'''

import argparse
import copyit
import masscopy
import ififuncs

def parse_args():
    '''
    Accepts command line arguments.
    '''
    parser = argparse.ArgumentParser(
        description='Performs copyit.py in a batch'
        'Launches multiple copyit jobs. This is different to masscopy.py,'
        'which takes a single directory as input and launches multiple copyit jobs.'
        'This script takes multiple inputs and copies them to the output directory.'
        ' Written by Kieran O\'Leary.')
    parser.add_argument(
        '-i', nargs='+',
        help='full path of input directory',
        required=True
    )
    parser.add_argument(
        '-o',
        help='full path of output directory',
        required=True)
    parser.add_argument(
        '-l', '-lto',
        action='store_true',
        help='use gcp instead of rsync on osx for SPEED on LTO')
    parser.add_argument(
        '-y',
        action='store_true',
        help='Answers YES to the question: Not enough free space, would you like to continue?'
    )
    args = parser.parse_args()
    return args


def main():
    '''
    Launches functions
    '''
    log_names = []
    args = parse_args()
    desktop_logs_dir = ififuncs.make_desktop_logs_dir()
    for i in args.i:
        copyit_cmd = [i, args.o]
        if args.l:
            copyit_cmd.append('-l')
        elif args.y:
            copyit_cmd.append('-y')
        log_names.append(copyit.main(copyit_cmd))
    print('********\nWARNING - Please check the ifiscripts_logs directory on your Desktop to verify if ALL of your transfers were successful')
    masscopy.analyze_reports(log_names, desktop_logs_dir)



if __name__ == '__main__':
    main()



#!/usr/bin/env python3
'''
Allow sipcreator to batch process multiple folders in a directory
-dcp option is needed to process folders and grab html files
'''
import argparse
import os
import sys
import ififuncs
import sipcreator
from masscopy import analyze_log


def parse_args(args_):
    '''
    Parse command line arguments.
    '''
    parser = argparse.ArgumentParser(
        description='Launches sipcreator.py on multiple objects.'
        'Wraps objects into an Irish Film Institute SIP'
        ' Written by Kieran O\'Leary.'
    )
    parser.add_argument(
        '-i',
        help='full path of input folder - this folder should contain subfolders which in turn contain video files', required=True
    )
    parser.add_argument(
        '-o', '-output',
        help='full path of output directory', required=True
    )
    parser.add_argument(
        '-rename_uuid', action='store_true',
        help='Use with caution! This will rename an object with a randonly generated UUID'
    )
    parser.add_argument(
        '-user',
        help='Declare who you are. If this is not set, you will be prompted.'
    )
    parser.add_argument(
        '-l', action='store_true',
        help='invokes the -lto argument in copyit.py - uses gcp instead of rsync.'
    )
    parser.add_argument(
        '-d', '-dcp', action='store_true',
        help='Adds DCP specific processing, like creating objects subfolder with text extracted from <ContentTitleText> in the CPL.'
    )
    parser.add_argument(
        '-zip', action='store_true',
        help='Uses makezip.py to store the objects in an uncompressed ZIP'
    )
    parser.add_argument(
        '-oe',
        help='Enter the Object Entry number for the representation.SIP will be placed in a folder with this name.'
    )
    parser.add_argument(
        '-supplement_extension_pattern', nargs='+',
        help='Enter the filename extensions, seperated by spaces, which determine which files to be added to the supplemental subfolder within the metadata directory. For example -supplement_extension_pattern xml pdf txt docx will take all filenames with these extensions within your input directory and store them in metadata/supplemental.  Use this for information that supplements your preservation objects but is not to be included in the objects folder.'
    )
    parser.add_argument(
        '-object_extension_pattern', nargs='+',
        help='Enter the filename extensions, seperated by spaces, which determine which files to be added to the objects directory. For example -object_extension_pattern mxf stl will take all filenames with these extensions within your input directory and store them in objects.'
    )
    parser.add_argument(
        '-y',
        action='store_true',
        help='Answers YES to the question: Do you want to proceed? useful for unattended batches but not recommended without running -dryrun first'
    )
    parser.add_argument(
        '-dryrun', action='store_true',
        help='The script will reveal which identifiers will be assigned but will not actually perform any actions.'
    )
    parsed_args = parser.parse_args(args_)
    return parsed_args

def main(args_):
    ''''
    Launch all the functions for creating an IFI SIP.
    '''
    args = parse_args(args_)
    source_folder = args.i
    print(args)
    oe_dict = {}
    user = ififuncs.determine_user(args)
    if args.oe:
        object_entry = args.oe
    else:
        object_entry = ififuncs.get_object_entry()
    oe_digits = int(object_entry.replace('oe', ''))
    for folder in sorted(os.listdir(source_folder)):
        full_path = os.path.join(source_folder, folder)
        if os.path.isdir(full_path):
            try:
                folder_contents = os.listdir(full_path)
            except PermissionError:
                continue
            object_entry_complete = 'oe' + str(oe_digits)
            inputs = []
            supplements = []
            for files in folder_contents:
                if os.path.splitext(files)[1][1:].lower() in args.object_extension_pattern:
                    inputs.append(os.path.join(full_path, files))
                if os.path.splitext(files)[1][1:].lower() in args.supplement_extension_pattern:
                    supplements.append(os.path.join(full_path, files))
            if inputs:
                print(' - Object Entry: %s\n - Inputs: %s\n - Supplements: %s\n' % (object_entry_complete, inputs, supplements))
                oe_dict[object_entry_complete] = [inputs, supplements]
                oe_digits += 1
            else:
                print('Skipping %s as there are no files in this folder that match the -object_extension_pattern' % full_path)
    if args.dryrun:
        print('Exiting as you selected -dryrun')
        sys.exit()
    logs = []
    if args.y:
        proceed = 'Y'
    else:
        proceed = ififuncs.ask_yes_no(
            'Do you want to proceed?'
        )
    if proceed == 'Y':
        for sips in sorted(oe_dict):
            print(oe_dict[sips])
            sipcreator_cmd = ['-i',]
            for sipcreator_inputs in oe_dict[sips][0]:
                sipcreator_cmd.append(sipcreator_inputs)
            sipcreator_cmd += ['-supplement']
            for sipcreator_supplements in oe_dict[sips][1]:
                sipcreator_cmd.append(sipcreator_supplements)
            sipcreator_cmd += ['-user', user, '-oe', sips, '-o', args.o]
            if args.rename_uuid:
                sipcreator_cmd.append('-rename_uuid')
            if args.zip:
                sipcreator_cmd.append('-zip')
            if args.l:
                sipcreator_cmd.append('-l')
            print(sipcreator_cmd)
            sipcreator_log, _ = sipcreator.main(sipcreator_cmd)
            logs.append(sipcreator_log)
            for i in logs:
                if os.path.isfile(i):
                    print(("%-*s   : copyit job was a %s" % (50, os.path.basename(i), analyze_log(i))))
if __name__ == '__main__':
    main(sys.argv[1:])
    


#!/usr/bin/env python3
'''
Zips and verifies all files and folders within your input directory.
'''
from zipfile import ZipFile
import zipfile
import sys
import os
import time
import argparse
import subprocess
import datetime
import ififuncs

def parse_args(args_):
    '''
    Parse command line arguments.
    '''
    parser = argparse.ArgumentParser(
        description='Compress folders with ZIP and verifies the process'
        ' Written by Kieran O\'Leary.'
    )
    parser.add_argument(
        '-i', help='Input directory', required=True
    )
    parser.add_argument(
        '-o', help='Output directory', required=True
    )
    parser.add_argument(
        '-basename', help='Specify a basename for the output file. A basename is a filename without the full path eg output.zip'
    )
    parsed_args = parser.parse_args(args_)
    return parsed_args

def create_zip(source, destination, name):
    '''
    Creates an uncompressed zipfile for all files in the source directory
    and stores the zipfile in the destination directory
    '''
    pwd = os.getcwd()
    zip_start = datetime.datetime.now()
    full_zip = os.path.join(destination, name)
    os.chdir(os.path.dirname(source))

    # check if input folder size is greater than 100 gigs 100000000000
    if ififuncs.get_folder_size(source) > 100000000000:
        subprocess.call(['7za', 'a', '-tzip', '-v100g', '-mm=LZMA', '-mx=3', full_zip, os.path.basename(source)])
        full_zip = full_zip + '.001'
    else:
        subprocess.call(['7za', 'a', '-tzip', '-mm=LZMA', '-mx=3', full_zip, os.path.basename(source)])
    zip_finish = datetime.datetime.now()
    os.chdir(pwd)
    verify_start = datetime.datetime.now()
    print(' - Verifying the CRC32 checksums within the ZIP file..')
    if full_zip.endswith('.001'):
        try:
            result = subprocess.check_output(['7za', 't', full_zip,], stderr=subprocess.STDOUT).decode(sys.stdout.encoding)
        except subprocess.CalledProcessError as e:
            if 'Error' in e.output:
                result = 'FAILURE - Error Detected'
                print(e.output)
                print(result)
        if 'Everything is Ok' in result:
            print('Python has not detected any errors in your zipfile')
    else:
        with zipfile.ZipFile(full_zip, 'r') as myzip:
            result = myzip.testzip()
            if result is None:
                print('Python has not detected any errors in your zipfile')
            else:
                print('ERROR DETECTED IN %s '% result)
    verify_finish = datetime.datetime.now()
    total_zip = zip_finish - zip_start
    total_verify = verify_finish - verify_start
    print('Zipping duration = %s seconds' % (str(total_zip)))
    print('Verification duration = %s seconds' % (str(total_verify)))
    print('Total duration = %s seconds' % (str(total_zip + total_verify)))
    return result, full_zip

def main(args_):
    '''
    Zips and verifies all files and folders within your input directory.
    '''
    args = parse_args(args_)
    print(args)
    print('makezip.py started')
    source = args.i
    destination = args.o
    if args.basename:
        name = args.basename
    else:
        name = os.path.basename(source) + '.zip'
    result, full_zip = create_zip(source, destination, name)
    return result, full_zip

if __name__ == '__main__':
    ififuncs.check_existence(['7za'])
    main(sys.argv[1:])



#!/usr/bin/env python

from lxml import etree
import sys
import pdb
from glob import glob
import csv
import os
from os import listdir
from os.path import isfile, join
import subprocess
import base64
import time
import re
import smtplib
import argparse
import mimetypes
from email.mime.multipart import MIMEMultipart
from email import encoders
from email.message import Message
from email.mime.audio import MIMEAudio
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email.mime.text import MIMEText
try:
    import bagit
except ImportError:
    print 'skipping error'
ififuncs.check_existence(['ffprobe', 'ffmpeg'])
parser = argparse.ArgumentParser(description='DCP FIXITY checker/bagging tool.'
                                 ' Written by Kieran O\'Leary.')
parser.add_argument('input')
parser.add_argument(
                    '-bag', 
                    action='store_true',help='bag the dcp_dir if it passes the hash check')
parser.add_argument(
                    '-csv', 
                    action='store_true',help='File level csv is stored as sidecar to dcp directory')
parser.add_argument(
                    '-m', 
                    action='store_true',help='send email report')
args = parser.parse_args()

if args.bag:
    bagging = 'enabled'
else:
    bagging = 'disabled'

if args.m:
    email = 'enabled'
else:
    email = 'disabled'
#bagrm =  os.path.abspath('bag-rm.py') 
#bagit =  os.path.abspath('bagit.py') 
#print bagrm
dcp_dir = args.input
# Two csv functions. One to create a csv, the other to add info to.
def create_csv(csv_file, *args):
    f = open(csv_file, 'wb')
    try:
        writer = csv.writer(f)
        writer.writerow(*args)
    finally:
        f.close()
        
def append_csv(csv_file, *args):
    f = open(csv_file, 'ab')
    try:
        writer = csv.writer(f)
        writer.writerow(*args)
    finally:
        f.close()

# Create a new .csv file with headings.  
# CSV filename will be DCp directory name + time/date.

# CSV will be saved to your Desktop.


csv_report_filename = os.path.basename(dcp_dir) + '_dcp_level' + time.strftime("_%Y_%m_%dT%H_%M_%S")
# CSV will be saved to your Desktop.

csv_report = os.path.expanduser("~/Desktop/%s.csv") % csv_report_filename        
create_csv(csv_report, ('DCP NAME', 'DIRECTORY NAME', 'JUDGEMENT'))

if args.csv:
    csv_setting = 'enabled'

    
    
else:
    csv_setting = 'disabled'
    csv_filename = os.path.basename(dcp_dir) + '_file_level' + time.strftime("_%Y_%m_%dT%H_%M_%S")
    csvfile = os.path.expanduser("~/Desktop/%s.csv") % csv_filename
    #csv_file = os.path.expanduser("~/Desktop/%s.csv") % csv_filename
    create_csv(csvfile, ('MXF HASH', 'STORED HASH', 'FILENAME', 'JUDGEMENT'))
for root,dirnames,filenames in os.walk(dcp_dir):
    if ("ASSETMAP.xml"  in filenames) or ("ASSETMAP"  in filenames) :
        dir = root
        #print os.path.basename(os.path.dirname(root)) 
        filenoext = os.path.splitext(os.path.dirname(root))[0]
        #print filenoext + 'dfsdfjkljoewuiljkdfs'
        # Change directory to directory with video files

        if csv_setting == 'enabled': 
            #print filenoext
            # Generate new directory names in AIP
            fixity_dir   = "%s/fixity" % filenoext
            os.makedirs(fixity_dir)
            csvfile = fixity_dir + '/' + os.path.basename(os.path.dirname(root)) + '_item_level' + time.strftime("_%Y_%m_%dT%H_%M_%S") + '.csv'
            #csv_file = os.path.expanduser("~/Desktop/%s.csv") % csv_filename
            create_csv(csvfile, ('MXF HASH', 'STORED HASH', 'FILENAME', 'JUDGEMENT'))
        
        
        # Changing directory makes globbing easier (from my experience anyhow).
        os.chdir(dir)

        # Scan the main DCP directory for an assetmap.
        dcp_files = [f for f in listdir(dir) if isfile(join(dir, f))]
        if 'ASSETMAP' in dcp_files:
            assetmap = 'ASSETMAP'
        elif 'ASSETMAP.xml' in dcp_files:
            assetmap = 'ASSETMAP.xml'

        # Parse the assetmap in order to find the namespace.  
        try:  
            assetmap_xml = etree.parse(assetmap)
        except SyntaxError:
            append_csv(csvfile,('NOT A VALID ASSETMAP', 'NOT A VALID ASSETMAP', dir,'NOT A VALID ASSETMAP'))
            append_csv(csv_report,(os.path.basename(dir), dir, 'NOT A VALID ASSETMAP'))
            print 'not an assetmap!!!!'
            continue
           
        assetmap_namespace = assetmap_xml.xpath('namespace-uri(.)')

        

        # Get a list of all XML files in the main DCP directory.
        xmlfiles = glob('*.xml')

        # Generate an empty list as there may be multiple PKLs.
        pkl_list = []

        # Loop through xmlfiles in order to find any PKL files.
        for i in xmlfiles:
            try:  
                xmlname = etree.parse(i)
            except SyntaxError:
                append_csv(csvfile,('NOT A VALID PKL', 'NOT A VALID PKL', dir,'NOT A VALID PKL'))
                append_csv(csv_report,(os.path.basename(dir), dir, 'NOT A VALID PKL'))
                print 'not a valid PKL!!!!'
                continue
            except KeyError:
                append_csv(csvfile,('PKL APPEARS TO BE MISSING', 'PKL APPEARS TO BE MISSING', dir,'PKL APPEARS TO BE MISSING'))
                append_csv(csv_report,(os.path.basename(dir), dir, 'PKL APPEARS TO BE MISSING'))
                print 'Missing PKL!!!!'
                continue
            
            is_pkl = xmlname.xpath('namespace-uri(.)')
            if 'PKL' in is_pkl:
                pkl_list.append(i)
            
        if len(pkl_list) == 0:
            
            append_csv(csvfile,('PKL APPEARS TO BE MISSING', 'PKL APPEARS TO BE MISSING', dir,'PKL APPEARS TO BE MISSING'))
            append_csv(csv_report,(os.path.basename(dir), dir, 'PKL APPEARS TO BE MISSING'))
            continue
        
              
        # Generate an empty dictionary that will link the PKL hashes to each UUID.        
        pkl_hashes = {}

        # Loop through the PKLs and link each hash to a UUID.
        for i in pkl_list: 
            pkl_parse = etree.parse(i)
            pkl_namespace = pkl_parse.xpath('namespace-uri(.)') 
            hashes =  pkl_parse.findall('//ns:Hash',namespaces={'ns': pkl_namespace})
            xmluuid =  pkl_parse.findall('//ns:Asset/ns:Id',namespaces={'ns': pkl_namespace})

            counter = 0
            
            while counter <= len(hashes) -1 : # The -1 is there because of lxml's zero indexing.
                pkl_hashes[xmluuid[counter].text] = hashes[counter].text # {pkl_uuid:pkl_hash}
                counter +=1
                
        # Begin analysis of assetmap xml.
        counter = 0
        assetmap_paths =  assetmap_xml.findall('//ns:Path',namespaces={'ns': assetmap_namespace})
        assetmap_uuids =  assetmap_xml.findall('//ns:Asset/ns:Id',namespaces={'ns': assetmap_namespace})
        #while counter <= len(assetmap_paths) -1 :
            
        counter = 0

        file_paths = {}
        
        while counter <= len(assetmap_paths) -1 :
            if 'file:///' in assetmap_paths[counter].text:
                remove_this = 'file:///'
                assetmap_paths[counter].text =  assetmap_paths[counter].text.replace(remove_this,"")
            elif 'file://' in assetmap_paths[counter].text:
                remove_this = 'file://'
                assetmap_paths[counter].text =  assetmap_paths[counter].text.replace(remove_this,"")            

            elif 'file:/' in assetmap_paths[counter].text:
                remove_this = 'file:/'
                assetmap_paths[counter].text =  assetmap_paths[counter].text.replace(remove_this,"")
            

            file_paths[assetmap_uuids[counter].text] = [assetmap_paths[counter].text] # {assetmapuuid:assetmapfilename}
            counter +=1

        # Removes PKLs from list of files to hash, as these files are not in manifest.
        keys_to_remove = [] 

        for i in file_paths:
            if file_paths[i][0] in pkl_list:
                keys_to_remove.append(i)
        # PKL files are deleted from the file_paths dictionary. 
        for i in keys_to_remove:
            del file_paths[i]       

        # Check if there are any files missing from the DCP.

        missing_files = []
        for i in file_paths:
            if not os.path.isfile(file_paths[i][0]): # This checks if the file exists.
                print time.strftime("%Y-%m-%dT%H:%M:%S") + ' - **********' + file_paths[i][0] + ' is missing **********'
                missing_files.append(i)
                # Add missing file info to the csv.
                append_csv(csvfile,('MISSING FILE', pkl_hashes[i], os.path.abspath(file_paths[i][0]),'MISSING FILE'))

        # This removes the missing files from the hashable list. 
        for i in missing_files:
            del file_paths[i]
            del pkl_hashes[i]

        # Generate fresh hashes on the actual files in the DCP.           
        for i in file_paths:  
            print time.strftime("%Y-%m-%dT%H:%M:%S") + ' - Generating fresh hash for ' + file_paths[i][0]
            # Create SHA-1 binary hashes with OPENSSL.
            
            openssl_hash = subprocess.check_output(['openssl',
                                                    'sha1',
                                                    '-binary',
                                                     file_paths[i][0]])
            # Encode the hashes as base64.
            b64hash =  base64.b64encode(openssl_hash)
            # Append hashes to the list within the file_paths dictionary. 
            file_paths[i].append(b64hash)

        # Compare the hashes in the PKL manifest to the fresh hashes. 
        hash_mismatches = []       
        for i in file_paths:
            if file_paths[i][1] == pkl_hashes[i]:
                print file_paths[i][0] + ' is ok'
                append_csv(csvfile,(file_paths[i][1], pkl_hashes[i], os.path.abspath(file_paths[i][0]),'HASH MATCH'))
            else:
                print file_paths[i][0] + ' mismatch'
                hash_mismatches.append(file_paths[i][0])
                append_csv(csvfile,(file_paths[i][1], pkl_hashes[i], os.path.abspath(file_paths[i][0]),'HASH MISMATCH'))
            
                
        if len(hash_mismatches) > 0:
            report = ' but THERE ARE HASH MISMATCHES. SCROLL UP FOR MORE INFO OR CHECK THE CSV'
            baggable = 'n'
            print 'This DCP will not be bagged as it could not pass a fixity check'
        else:
            report = ' and all hashes match.'
            baggable = 'y'

        if len(missing_files) > 0:
            print time.strftime("%Y-%m-%dT%H:%M:%S") + ' - WARNING - THERE ARE FILES MISSING FROM THIS DCP. SCROLL UP FOR MORE INFO OR CHECK THE CSV'
            append_csv(csv_report,(os.path.dirname(root), dir, 'FILES MISSING - CHECK REPORT'))
            print 'This DCP will not be bagged as it could not pass a fixity check'
        else: 
            print time.strftime("%Y-%m-%dT%H:%M:%S") + ' - All files are present in your DCP' + report
            append_csv(csv_report,(os.path.dirname(root), dir,'All files present ' + report))
            
        if bagging == 'enabled':
            if baggable == 'y':
                #pdb.set_trace()
                if os.path.dirname(root) != args.input:
                   

                    dir = os.path.dirname(root)
                    
                    if args.input == os.path.dirname(dir):
                        os.chdir(dir)

                        bag = bagit.make_bag(dir)
                        
                    else:
                        print 'bagging not supported for this folder structure right now'
                else:
                     
                     print 'bagging not supported for this folder structure right now'
if email == 'enabled': 
    emailfrom = ""
    emailto = ['', '']
    #emailto = ", ".join(emailto)
    fileToSend = csv_report
    username = ""
    password = ""

    msg = MIMEMultipart()
    msg["From"] = emailfrom
    msg["To"] = ", ".join(emailto)
    msg["Subject"] = "Hash check complete"
    msg.preamble = "testtesttest"
    body = MIMEText("example email body")
    msg.attach(body)

    ctype, encoding = mimetypes.guess_type(fileToSend)
    if ctype is None or encoding is not None:
        ctype = "application/octet-stream"

    maintype, subtype = ctype.split("/", 1)

    if maintype == "text":
        fp = open(fileToSend)
        # Note: we should handle calculating the charset
        attachment = MIMEText(fp.read(), _subtype=subtype)
        fp.close()
    elif maintype == "image":
        fp = open(fileToSend, "rb")
        attachment = MIMEImage(fp.read(), _subtype=subtype)
        fp.close()
    elif maintype == "audio":
        fp = open(fileToSend, "rb")
        attachment = MIMEAudio(fp.read(), _subtype=subtype)
        fp.close()
    else:
        fp = open(fileToSend, "rb")
        attachment = MIMEBase(maintype, subtype)
        attachment.set_payload(fp.read())
        fp.close()
        encoders.encode_base64(attachment)
    attachment.add_header("Content-Disposition", "attachment", filename=fileToSend)
    msg.attach(attachment)


    server_ssl = smtplib.SMTP_SSL("smtp.gmail.com", 465)
    server_ssl.ehlo() # optional, called by login()
    server_ssl.login(username, password)  
    # ssl server doesn't support or need tls, so don't call server_ssl.starttls() 
    server_ssl.sendmail(emailfrom, emailto, msg.as_string())
    print msg.as_string()
    #server_ssl.quit()
    server_ssl.close()
    print 'successfully sent the mail'    



#!/usr/bin/env python
'''
Checks packages for correct structure.
Also returns a dictionary containing important varibles such as
folder paths and identifiers.
'''
import os
import argparse
import sys
import ififuncs


def parse_args(args_):
    '''
    Parse command line arguments.
    '''
    parser = argparse.ArgumentParser(
        description='Check packages for correct structure.'
        ' Written by Kieran O\'Leary.'
    )
    parser.add_argument(
        'input',
        help='full path of package'
    )
    parsed_args = parser.parse_args(args_)
    return parsed_args


def analyse_package(sip_path):
    '''
    Return a dictionary full of folder and file path variables
    '''
    package_info = {}
    package_info['sip_path'] = sip_path
    uuid = os.path.basename(sip_path)
    if ififuncs.validate_uuid4(uuid) is not False:
        package_info['uuid'] = uuid
        package_info['oe_path'] = os.path.dirname(sip_path)
        package_info['parent_identifier'] = os.path.basename(package_info['oe_path']) # can be OE or aaa
        package_info['manifest'] = os.path.join(package_info['oe_path'], uuid + '_manifest.md5')
        package_info['logs_dir'] = os.path.join(sip_path, 'logs')
        package_info['metadata_dir'] = os.path.join(sip_path, 'metadata')
        package_info['objects_dir'] = os.path.join(sip_path, 'objects')
        package_info['sip_log'] = os.path.join(package_info['logs_dir'], uuid + '_sip_log.log')
        package_info['objects'] = os.listdir(package_info['objects_dir'])
    else:
        return False
    return package_info

def test_package(package_info):
    '''
    Test if key files or folders exist.
    '''
    error_list = []
    question_list = []
    folder_list = [
        package_info['oe_path'],
        package_info['logs_dir'],
        package_info['objects_dir'],
        package_info['metadata_dir'],
        package_info['sip_path']
    ]
    file_list = [
        package_info['manifest'],
        package_info['sip_log']
    ]
    for folder in folder_list:
        if not os.path.isdir(folder):
            error_list.append(folder)
    for filename in file_list:
        if not os.path.isfile(filename):
            error_list.append(filename)
    metadata_files = os.listdir(package_info['metadata_dir'])
    for metadata_file in metadata_files:
        full_path = os.path.join(package_info['metadata_dir'], metadata_file)
        if os.path.getsize(full_path) == 0:
            question_list.append(full_path)
    for object_files in package_info['objects']:
        mediainfo = os.path.join(package_info['metadata_dir'], object_files + '_mediainfo.xml')
        if not os.path.isfile(mediainfo):
             question_list.append(mediainfo)
        mediatrace= os.path.join(package_info['metadata_dir'], object_files + '_mediatrace.xml')
        if not os.path.isfile(mediatrace):
             question_list.append(mediatrace)
    if not error_list:
        print('Key files and folders all exist')
    else:
        for error in error_list:
            print((('%s does not exist') % error))
    if not question_list:
        print('No further errors detected')
    else:
        for error in question_list:
            print((('%s requires investigation') % error))
    return error_list, question_list


def main(args_):
    '''
    Launch all the functions for determining folder and file paths.
    '''
    args = parse_args(args_)
    source = args.input
    sip_path = ififuncs.check_for_sip([source])
    if sip_path is not None:
        package_info = analyse_package(sip_path)
    if package_info is False:
        print('Valid UUID not found in folder path')
    else:
        print(package_info)
    error_list, question_list = test_package(package_info)
    return package_info, error_list, question_list

if __name__ == '__main__':
    main(sys.argv[1:])



#!/usr/bin/env python3
'''
Generates a helper accessions register based on the metadata in other
spreadsheets.
'''
import sys
import argparse
import csv
import time
import os
import ififuncs

def parse_args(args_):
    '''
    Parse command line arguments.
    '''
    parser = argparse.ArgumentParser(
        description='Generates a helper accessions register.'
        ' Written by Kieran O\'Leary.'
    )
    parser.add_argument(
        '-sorted_csv', help='Path to sorted helper CSV'
    )
    parser.add_argument(
        '-pbcore_csv',
        help='Path to technical/PBCore CSV.'
    )
    parser.add_argument(
        '-filmo_csv',
        help='Path to Filmographic CSV. Must contain reference numbers.'
    )
    parsed_args = parser.parse_args(args_)
    return parsed_args


def find_checksums(csv_file, identifier):
    '''
    Finds the relevant entries in the CSV and prints to terminal
    '''
    csv_dict = ififuncs.extract_metadata(csv_file)
    manifest_lines = []
    for items in csv_dict:
        for x in items:
            if type(x) is dict:
                if identifier in x['path']:
                    identifier_string = "/%s/" % identifier
                    manifest_line = x['hash_code'] + '  ' + x['path'].replace(identifier_string, '')
                    manifest_lines.append(manifest_line)
    strongbox_list = sorted(manifest_lines, key=lambda x: (x[130:]))
    return strongbox_list

def main(args_):
    '''
    Launches functions that will generate a helper accessions register
    '''
    args = parse_args(args_)
    sorted_csv_dict = ififuncs.extract_metadata(args.sorted_csv)[0]
    pbcore_csv_dict = ififuncs.extract_metadata(args.pbcore_csv)[0]
    filmo_csv_dict = ififuncs.extract_metadata(args.filmo_csv)[0]
    for accession in sorted_csv_dict:
        number = accession['accession number']
        for technical_record in pbcore_csv_dict:
            if technical_record['Accession Number'] == number:
                accession['acquisition method'] = technical_record['Type Of Deposit']
                accession['acquired from'] = technical_record['Donor']
                accession['date acquired'] = technical_record['Date Of Donation']
                for filmographic_record in filmo_csv_dict:
                    if filmographic_record['Filmographic URN'] == technical_record['Reference Number']:
                        if filmographic_record['Title/Name'] == '':
                            title = filmographic_record['Series Title'] + '; ' + filmographic_record['Episode No']
                        else:
                            title = filmographic_record['Title/Name']
                        simple = '%s (%s) | %s' % (title, filmographic_record['Year'], technical_record['dig_object_descrip'])
                        if accession['acquisition method'] == 'Reproduction':
                            simple += ' | Reproduction of %s' % technical_record['TTape Origin']
                        accession['simple name; basic description; identification; historical information'] = simple
    desktop_logs_dir = ififuncs.make_desktop_logs_dir()
    new_csv_filename = time.strftime("%Y-%m-%dT%H_%M_%S_") + 'helper_register.csv'
    new_csv = os.path.join(desktop_logs_dir, new_csv_filename)
    with open(new_csv, 'w', encoding='utf-8') as csvfile:
            fieldnames = ififuncs.extract_metadata(args.sorted_csv)[1]
            writer = csv.DictWriter(csvfile, fieldnames=fieldnames)
            writer.writeheader()
            for i in sorted_csv_dict:
                writer.writerow(i)
    print('\nYour helper CSV file is located here: %s\n' % new_csv)
    return new_csv



if __name__ == '__main__':
    main(sys.argv[1:])



#!/usr/bin/env python3
'''
Launches copyit.py for subfolders that have md5 anifests.
'''
import os
import argparse
import time
import copyit
from ififuncs import make_desktop_logs_dir


def analyze_log(logfile):
    '''
    Analyzes logfiles on the desktop and summarises the outcome.
    '''
    outcome = ''
    with open(logfile, 'r', encoding='utf-8') as fo:
        log_lines = fo.readlines()
        for line in log_lines:
            if 'EVENT = File Transfer Judgement - Success' in line:
                outcome = 'success'
            if 'EVENT = File Transfer Outcome - Failure' in line:
                outcome = 'failure'
            if 'EVENT = Existing source manifest check - Failure' in line:
                outcome = 'failure - might be outdated manifests in use'
        return outcome

def parse_args():
    '''
    Accepts command line arguments.
    '''
    parser = argparse.ArgumentParser(
        description='Performs moveit.py in a batch'
        ' Written by Kieran O\'Leary.')
    parser.add_argument(
        'input',
        help='full path of input directory'
    )
    parser.add_argument(
        '-o',
        help='full path of output directory',
        required=True)
    parser.add_argument(
        '-l', '-lto',
        action='store_true',
        help='use gcp instead of rsync on osx for SPEED on LTO')
    parser.add_argument(
        '-y',
        action='store_true',
        help='Answers YES to the question: Not enough free space, would you like to continue?'
    )
    args = parser.parse_args()
    return args


def find_manifest(args):
    '''
    This function tries to find a manifest.
    It looks one folder beneath the input folder for a checksum manifest
    with a folder that has a matching name.
    '''
    # Creates an empty list called dirlist.
    dirlist = []
    # Creates a list of items in your input called directory_contents.
    directory_contents = sorted(os.listdir(args.input))
    # lists all contents of your input and analyzes each one in a `for loop`.
    for item in directory_contents:
        full_path = os.path.join(args.input, item)
        manifest = os.path.join(
            args.input, item
        ) + '_manifest.md5'
        # checks if the manifest exists
        if os.path.isfile(manifest):
            #if the manifest exists, add to dirlist
            dirlist.append(full_path)
        # checks if each item is a directory.
        if os.path.isdir(full_path):
            directories = os.listdir(full_path)
            for subdirectories in directories:
                full_subdirectory_path = os.path.join(
                    full_path, subdirectories
                )
                if os.path.isdir(full_subdirectory_path):
                    manifest = os.path.join(
                        os.path.dirname(
                            full_subdirectory_path
                        ), subdirectories + '_manifest.md5'
                        )
                    if os.path.isfile(manifest):
                        dirlist.append(os.path.dirname(full_subdirectory_path))
    return dirlist # the dirlist is sent back out to the rest of the script.


def analyze_reports(log_names, desktop_logs_dir):
    '''
    Tries to locate copyit.py logs on the desktop and analyzes them.
    '''
    print(' - SUMMARY REPORT')
    for i in log_names:
        if os.path.isfile(i):
            print(" - %-*s   : %s" % (50, os.path.basename(i)[:-24], analyze_log(i)))
        else:
            print(' - %s can\'t find log file, trying again...' % i)
            for logs in os.listdir(desktop_logs_dir):
                # look at log filename minus the seconds and '.log'
                if os.path.basename(i)[:-7] in logs:
                    # make sure that the alternate log filename is more recent
                    if int(
                            os.path.basename(logs)[-12:-4].replace('_', '')) > int(os.path.basename(i)[-12:-4].replace('_', '')):
                        print(' - trying to analyze %s' % logs)
                        print(" - %-*s   : %s" % (50, os.path.basename(logs)[:-24], analyze_log(os.path.join(desktop_logs_dir, logs))))


def main():
    '''
    Launches the other functions wihch attempt to run multiple copyit.py
    instances if manifests and matching sidecar directories are found
    inside of the input directory.
    '''
    args = parse_args()
    all_files = find_manifest(args)
    processed_dirs = []
    log_names = []
    print('\n\n - **** All of these folders will be copied to %s\n' % args.o)
    for i in all_files:
        absolute_path = os.path.join(args.o, os.path.basename(i))
        if os.path.isdir(absolute_path):
            print(' - %s already exists, skipping' % absolute_path)
        else:
            print(' - %s will be copied' % i)
    time.sleep(2)
    for i in all_files:
        absolute_path = os.path.join(args.o, os.path.basename(i))
        if os.path.isdir(absolute_path):
            print(' - %s already exists, skipping' % absolute_path)
        else:
            desktop_logs_dir = make_desktop_logs_dir()
            copyit_cmd = [os.path.join(args.input, i), args.o]
            if args.l:
                copyit_cmd.append('-l')
            elif args.y:
                copyit_cmd.append('-y')
            log_name = copyit.main(copyit_cmd)
            log_names.append(log_name)
            processed_dirs.append(os.path.basename(os.path.join(args.input, i)))
            print(' - ********\nWARNING - Please check the ifiscripts_logs directory on your Desktop to verify if ALL of your transfers were successful')
            analyze_reports(log_names, desktop_logs_dir)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
'''
Specific workflow for the IFI Irish Film Archive's Special Collections.
'''
import argparse
import os
import shutil
from ififuncs import hashlib_manifest
from ififuncs import generate_log

def count_files(source):
    '''
    Counts the files to be processed.
    '''
    file_count = 1
    for _, directories, filenames in os.walk(source):
        filenames = [f for f in filenames if f[0] != '.']
        directories[:] = [d for d in directories if d[0] != '.']
        # Surely just len() would suffice here?
        for files in filenames:
            print "Calculating number of files in all subdirectories -  %s files        \r"% file_count,
            file_count += 1
    return file_count


def make_parser():
    '''
    Accepts command line arguments.
    '''
    parser = argparse.ArgumentParser(
        description='Batch MD5 checksum generator.'
        'Accepts a parent folder as input and will generate manifest for each subfolder.'
        ' Designed for a specific IFI Irish Film Archive workflow. '
        'Written by Kieran O\'Leary.'
    )
    parser.add_argument(
        'input', help='file path of parent directory'
    )
    parser.add_argument(
        '-v', action='store_true',
        help='verbose mode - some extra information such as overall file count.'
    )
    return parser


def create_manifest(source):
    '''
    Generates a master log and creates checksum manifests for all subdirectories.
    '''
    master_log = os.path.expanduser('~/Desktop/batchfixity_errors.log')
    os.chdir(source)
    for dirname in os.walk('.').next()[1]:
        full_path = os.path.join(source, dirname)
        manifest_textfile = '%s/%s_manifest.md5' % (full_path, dirname)
        if not os.path.isfile(manifest_textfile):
            log_name = '%s/%s_fixity.log' % (
                os.path.dirname(full_path), dirname
            )
            generate_log(log_name, 'batchfixity started')
            generate_log(log_name, '%s created' % manifest_textfile)
            try:
                hashlib_manifest(full_path, manifest_textfile, full_path)
                generate_log(log_name, 'manifest creation complete')
                shutil.move(log_name, full_path)
            except IOError:
                with open(master_log, 'ab') as log:
                    log.write(
                        '%s has failed probably because of special characters like a fada\n' % full_path
                    )
                    generate_log(
                        log_name, 'manifest has failed probably because of special characters like a fada'
                        )


def main():
    '''
    Launches the main functions.
    '''
    parser = make_parser()
    args = parser.parse_args()
    if args.v:
        count_files(args.input)
    create_manifest(args.input)


if __name__ == '__main__':
    main()

